<?php

namespace WpBeacon\Actions;

use WP_Application_Passwords;
use WP_User;
use WpBeacon\Providers\MonitoringUserProvider;
use WpBeacon\Support\AdminNotice;

class MonitoringUserAction
{
    const PASSWORD_LENGTH = 42;
    const APP_PASSWORD_NAME = 'Cyrano Monitoring Application Password';
    const ERROR_CREATING_USER = 'Error creating monitoring user.';
    const ERROR_CREATING_PASSWORD = 'Error creating application password.';

    public static function updateOrCreate()
    {
        $name = MonitoringUserProvider::USERNAME;
        $email = MonitoringUserProvider::EMAIL;
        $capability = MonitoringUserProvider::CAPABILITY;

        $userId = email_exists($email);
        if (!$userId) {
            self::createUser($name, $email, $capability);
            return;
        }

        self::setSystemUserCapabilities($userId, $capability);
    }

    private static function createUser($name, $email, $capability)
    {
        $password = wp_generate_password(self::PASSWORD_LENGTH);
        $userId = wp_create_user($name, $password, $email);

        if (is_wp_error($userId)) {
            AdminNotice::error(self::ERROR_CREATING_USER);
            return;
        }

        self::setSystemUserCapabilities($userId, $capability);
        self::createApplicationPassword($userId, $name);
    }

    private static function createApplicationPassword($userId, $name)
    {
        $appPasswordData = [
            'name' => self::APP_PASSWORD_NAME
        ];
        $appPassword = WP_Application_Passwords::create_new_application_password($userId, $appPasswordData);

        if (is_wp_error($appPassword)) {
            AdminNotice::error(self::ERROR_CREATING_PASSWORD);
            return;
        }

        $notice = "Zugangsdaten fürs Cyrano WordPress Monitoring:<br>Benutzername: <code>$name</code><br>Passwort: <code>{$appPassword[0]}</code>";
        AdminNotice::success($notice);
    }

    public static function setSystemUserCapabilities(int $userId, string $capability): void
    {
        $user = new WP_User($userId);
        $user->description = 'System user: ' . MonitoringUserProvider::USERNAME;
        $user->set_role('');

        $user->add_cap($capability);
        foreach ($user->allcaps as $cap => $value) {
            if ($cap != $capability) {
                $user->remove_cap($cap);
            }
        }
    }
}
